#!/usr/bin/env python3

import logging
import argparse
import os
import sys
import json
from ollama_data_tools import ollama_data as od

def get_args():
    """
    Parse and return command line arguments.

    :return: Parsed command line arguments.
    """
    script_name = os.path.basename(sys.argv[0])
    parser = argparse.ArgumentParser(
        description='Search over Ollama models.',
        formatter_class=argparse.RawDescriptionHelpFormatter,
        epilog=f"""
Examples:

  The following JMESPath query (http://jmespath.org) shows the largest model:

    ./{script_name} "max_by(@, &total_weights_size).{{name: name, size: total_weights_size}}"

  The following example shows how to use a regex pattern to filter models:

    ./{script_name} --regex "mistral:latest" --regex-path name "[*].{{name: name, size: total_weights_size}}"

  You can also pipe a query from a file or another command:

    cat query.txt | ./{script_name}

  Using regex and regex-path with a piped query:

    echo "[*].{{info: {{ name: name, other: weights}}}}" | ./{script_name} --regex 14f2 --regex-path "info.other[*].file_name"
"""
    )

    parser.add_argument('--schema',
                        help='Print the JSON schema.',
                        action='store_true')

    parser.add_argument('--regex',
                        help='Regular expression to match.',
                        metavar='REGEX')
    
    parser.add_argument('--regex-path',
                        help='The JMESPath query for the regex pattern to apply against.',
                        metavar='QUERY',
                        default='@')

    parser.add_argument('--debug', 
                        help='Set logging level to DEBUG.',
                        action='store_true')
    
    parser.add_argument('query',
                        help='The JMESPath query to filter results.',
                        nargs='?', 
                        default=None)

    parser.add_argument('--cache-time',
                        help='Time to keep the cache file.',
                        metavar='STRING',
                        type=str, 
                        default="1 hour")

    parser.add_argument('--cache-path',
                        help='The path to the cache file.',
                        metavar='PATH',
                        default='~/.ollama_data/cache')

    return parser.parse_args()

def main():
    """
    Main function to execute the script.
    """
    args = get_args()

    logging.basicConfig(level=logging.DEBUG if args.debug else logging.INFO)
    logger = logging.getLogger(__name__)
    logger.debug(f"Arguments received: {args}")

    # If no query is provided as an argument, check if a query is being piped in
    query = args.query
    if not query and not sys.stdin.isatty():
        query = sys.stdin.read().strip()
        logger.debug(f"Piped query received: {query}")

    if not query and not args.schema:
        query = '[*]'

    if args.schema:
        print(json.dumps(od.OllamaData.get_schema(), indent=4))
        sys.exit(0)

    data = od.OllamaData(cache_path=args.cache_path,
                         cache_time=args.cache_time)
    output = data.search(
        query=query,
        regex=args.regex,
        regex_path=args.regex_path)

    print(json.dumps(output, indent=4))

if __name__ == "__main__":
    main()



#!/usr/bin/env python3

import re
from ollama_data_tools import ollama_data as od
import subprocess
import argparse
from typing import Dict, Any, List
import sys
import logging

def run(args: List[str]) -> str:
    """
    Runs the given subprocess with the specified arguments.

    Args:
        args (List[str]): The arguments to pass to the subprocess.

    Returns:
        str: The output from the subprocess.
    """
    process = subprocess.Popen(args, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
    for line in iter(process.stdout.readline, b''):
        print(line.decode(), end='')

    process.stdout.close()
    process.wait()

    if process.returncode != 0:
        raise RuntimeError(f"Error: {process.stderr.read().decode()}")

def main():
    logging.basicConfig(level=logging.INFO)
    logger = logging.getLogger()

    script_name = sys.argv[0]
    parser = argparse.ArgumentParser(
        description='Adapts Ollama models for other engines (like `llamacpp`).',
        formatter_class=argparse.RawTextHelpFormatter,
        epilog="Example: " + script_name + " mistral llamacpp --engine-path /path/to/llamacpp --engine-args '--temp 0.5' '--n-gpu-layers 40' '--prompt \"[INST] You are a helpful AI assistant. [/INST]\"'")
    parser.add_argument('model', help='The model to run.', type=str, nargs='?')
    parser.add_argument('engine', help='The engine to use.', type=str, choices=['llamacpp'], default='llamacpp', nargs='?')
    parser.add_argument('--engine-path', help='The path to the engine.', type=str)
    parser.add_argument('--list-engines', help='List available engines.', action='store_true')
    parser.add_argument('--list-models', help='List available models.', action='store_true')
    parser.add_argument('--cache-path', help='The path to the cache file.', default='~/.ollama_data/cache')
    parser.add_argument('--cache-time', help='The time in seconds to keep the cache file.', type=str, default='1 day')
    parser.add_argument('--engine-args', help='Arguments to pass through to the engine.', nargs='*', default=[], type=str)
    parser.add_argument('--debug', help='Print debug information.', action='store_true')
    parser.add_argument('--show-template', help='Show the template for the model.', action='store_true')
    args = parser.parse_args()

    models = od.OllamaData(cache_path=args.cache_path, cache_time=args.cache_time)

    if args.debug:
        logger.setLevel(logging.DEBUG)

    def _llamacpp_args(options: Dict[str, Any]) -> List[str]:
        """
        Returns the arguments compatible with the llamacpp engine.

        Args:
            options (Dict[str, Any]): The options to use.

        Returns:
            List[str]: The arguments for the llamacpp engine.
        """

        if 'engine_path' not in options:
            raise ValueError("Engine path is required.")
        
        args = [
            options['engine_path'],
            '--model',
            options['model_path'],
            '--instruct'
        ]

        if options['engine_args']:
            for arg in options['engine_args']:
                match = re.match(r'--\w+ ["\'].*["\']', arg)
                if match:
                    key, value = arg.split(' ', 1)
                    value = value.strip('\'"')
                    args.extend([key, value])
                else:
                    args.extend(arg.split())

        return args

    engines = {
        'llamacpp': _llamacpp_args
    }

    if args.list_engines:
        print("Available engines:")
        for engine in engines.keys():
            print(f"  - {engine}")
        exit(0)

    if args.list_models:
        print("Available models:")
        model_names = models.search(query='[*].name')
        for model in model_names:
            print(f"  - {model}")
        exit(0)

    if not args.model:
        print("Specify a model to run or show the template for.")
        exit(1)

    model = models.get_model(args.model)
    if not model:
        print(f"Model '{args.model}' not found.")
        exit(1)

    if args.show_template:
        print("The template for the model has the following forms:")
        for line in model['template']:
            print(f"  - {line}")
        exit(0)

    if args.engine not in engines:
        print(f"Engine '{args.engine}' not found.")
        exit(1)

    options = {
        'engine_path': args.engine_path,
        'model_path': model['weights'][0]['file_path'],
        'engine_args': args.engine_args,
    }

    try:
        run_args = engines[args.engine](options)

        logger.debug(f"Running model '{args.model}' with engine '{args.engine}' using the following engine-args:")
        for arg in run_args:
            logger.debug(" - " + arg)
        run(run_args)
    except Exception as e:
        print(f"Error: {e}")
        exit(1)

if __name__ == "__main__":
    main()




import os
import json
from time import time
from typing import Dict, Any, Union
from ollama_data_tools import conversion_tools as ct

class JsonCache:
    """
    A class to manage a cache file on disk. It uses JSON to store the data.
    It is a very simple cache that operates at the granularity of a
    single JSON or dictionary object. It is an immutable cache, meaning
    that once the data is saved, it cannot be modified. If you need to
    modify the data, you must save a new version of the data.

    Example usage:

        cache = JsonCache('~/.cache/my_cache.json', duration='5 minutes')
        cache.save({'key': 'value'})
        data = cache.load()

    It is designed to allow for slow operations to be cached between runs,
    say for a CLI tool that queries a remote API. The cache file is
    considered invalid if it is older than the specified duration.
    """

    def __init__(self, path: str, duration: Union[int,str] = '1 week'):
        """
        Initialize the CacheManager object.

        :param path: The path to the cache file.
        :param duration: The time duration the cache is valid. It can be a string
                         like "1 day" or an integer representing the number of seconds.
        """
        if not isinstance(duration, str):
            duration = f"{duration} seconds"

        _, duration = ct.parse_duration(duration)
        self.duration = duration
        self.path = os.path.expanduser(path)

        if self.duration.total_seconds() <= 0:
            raise ValueError("Duration must be positive.")

    def get_time_remaining(self) -> float:
        """
        Get the time remaining before the cache expires.

        :return: The time remaining in seconds.
        """
        if not os.path.exists(self.path):
            return 0
        elapsed = time() - os.path.getmtime(self.path)
        return max(0, self.duration.total_seconds() - elapsed)


    def is_valid(self) -> bool:
        """
        Check if the cache file is valid based on the elapsed time since
        it was created.

        :return: True if the cache is valid, False otherwise.
        """
        if not os.path.exists(self.path):
            return False

        elapsed = time() - os.path.getmtime(self.path)
        return elapsed <= self.duration.total_seconds()
    
    def clear(self) -> None:
        """
        Clear the cache file.
        """
        if os.path.exists(self.path):
            os.remove(self.path)

    def load(self) -> Dict[str, Any]:
        """
        Load the cache file from disk.

        :return: The content of the cache file as a dictionary.
        """
        if not self.is_valid():
            raise RuntimeError("Cache is invalid.")
        
        with open(self.path, 'r') as file:
            return json.load(file)

    def save(self, data: Dict[str, Any]) -> None:
        """
        Save data to the cache file.

        :param data: The data to cache.
        """
        dir_name = os.path.dirname(self.path)
        if not os.path.exists(dir_name):
            os.makedirs(dir_name)

        with open(self.path, 'w') as file:
            json.dump(data, file, indent=4)



import jmespath
from ollama_data_tools import ollama_data_utils as odu
from ollama_data_tools import regex_path_matcher
from typing import List, Dict, Any, Optional
from ollama_data_tools import json_cache as cm

class OllamaData:
    @staticmethod
    def get_schema() -> Dict[str, Any]:
        """
        Returns the schema of the `OllamaData` object.

        :return: An object representing the schema.
        """
        return odu.get_schema()

    def __init__(self,
                 cache_path: str = '~/.ollama_data/cache',
                 cache_time: str = '1 day'):
        """
        Initialize the OllamaData object.

        :param cache_path: The path to the cache file.
        :param cache_time: The duration the cache is valid.
        """

        self.cache = cm.JsonCache(cache_path, cache_time)

    def __len__(self) -> int:
        """
        Get the number of models.

        :return: The number of models.
        """
        return len(self.get_models())

    def __getitem__(self, index: int) -> Dict[str, Any]:
        """
        Get a model by index.

        :param index: The index of the model.
        :return: A dictionary representing the model.
        """
        models = self.get_models()
        if index < 0 or index >= len(models):
            raise IndexError("Index out of range")
        return models[index]

    def get_model(self, name: str) -> Dict[str, Any]:
        """
        Get the model by name (starts with the name and returns
        the most specific model). The total order is given by:

        1. The model with the shortest name that starts with the given name.
        2. If there are multiple models with the same length, we return the first one.
        3. If a model is named `<model_name>:latest`, we compute its length
           without the `:latest` suffix unless `name` is `<model_name>:latest`.

        :param name: The name of the model.
        :return: A dictionary representing the model.
        """
        
        models = [m for m in self.get_models() if m['name'].startswith(name)]
        if not models:
            raise ValueError(f"No model with name '{name}' found")
        
        def _len(m):
            if 'name' not in m:
                return int(1e9)
            l = len(m['name'])
            if m['name'].endswith(':latest'):
                l -= len(':latest')
            return l
        return min(models, key=_len)     


    def get_models(self) -> Dict[str, Any]:
        """
        Get the models. We either have it in memory and it has not expired,
        or it has expired or it is not already in memory and we regenerate the
        model data.

        :return: A dictionary representing the models. See `get_schema` for the schema.
        """
        if not self.cache.is_valid():
            self.cache.save(odu.get_models())

        return self.cache.load()

    def search(self,
               query: str = '[*]',
               regex: Optional[str] = None,
               regex_path: str = '@') -> Dict[str, Any]:
        """
        Query/search/view the models using a JMESPath query, regex filter, and
        exclude keys.

        :param query: The JMESPath query to filter and provide a view of the models.
        :param regex: The regex pattern to match against the output.
        :param regex_path: The JMESPath query for the regex pattern. See
                           `utils.regex_path_matcher` for more information.
        :return: JSON (dict) object representing some view of the models.
        """
        output = jmespath.search(query, self.get_models())
        if regex:
            output = regex_path_matcher.regex_path_matcher(output, regex, regex_path)
        return output



#!/usr/bin/env python3

import os
from ollama_data_tools import ollama_data as od
import subprocess
import logging
import argparse
import json
import sys

# Configure logging
logging.basicConfig(level=logging.INFO)
logger = logging.getLogger()

def export_model(model, outdir, hash_length):
    """
    Exports a model to the specified output directory by creating soft links for the weights and saving the metadata.

    Args:
        model (dict): The model data.
        outdir (str): The output directory where the model will be exported.
    """
    link = os.path.join(outdir, model['name'])

    for weight in model['weights']:
        weight_link = link
        if hash_length > 0:
            weight_link += "_" + weight['hash'][:hash_length]
        result = subprocess.run(['ln', '-s', weight['file_path'], weight_link])
        if result.returncode != 0:
            logger.error(f"Error creating weight soft-link for {weight_link}")
        
        weight['soft-link'] = weight_link
        logger.debug(f"Model weight {weight['file_path']} exported to soft-link: {weight_link}")
    
    meta_link = link + '.json'
    with open(meta_link, 'w') as f:
        json.dump(model, f, indent=4)
    logger.debug(f"Model {model['name']} metadata exported to {meta_link}")

def main():
    # Parse command-line arguments
    parser = argparse.ArgumentParser(description='Export Ollama models to a self-contained directory.')
    parser.add_argument('--models', help='Comma-separated list of models to export.', nargs='?')
    parser.add_argument("outdir", help="The output directory.", type=str)
    parser.add_argument("--cache-path", help="The cache path.", default="~/.ollama_data/cache")
    parser.add_argument("--cache-time", help="The cache time.", default='1 day')
    parser.add_argument("--debug", help="Enable debug logging.", action='store_true')
    parser.add_argument("--hash-length", help="The length of the hash to use for the weight soft-links.", default=8, type=int)
    args = parser.parse_args()

    # Set logging level to DEBUG if --debug flag is provided
    if args.debug:
        logger.setLevel(logging.DEBUG)

    # Initialize OllamaData
    ollama_data = od.OllamaData(cache_path=args.cache_path, cache_time=args.cache_time)

    # Determine model names to export
    if args.models:
        model_names = args.models.split(',')
    else:
        # Check if data is being piped into stdin
        if not sys.stdin.isatty():
            model_names = [line.strip() for line in sys.stdin]
        else:
            model_names = ollama_data.search("[*].name")

    # Check if the output directory already exists
    if os.path.exists(args.outdir):
        logger.error(f"Output directory {args.outdir} already exists.")
        sys.exit(1)

    logger.debug(f"Exporting {len(model_names)} models.")

    # Create the output directory
    os.makedirs(args.outdir)

    # Export each model
    for model_name in model_names:
        model = ollama_data.get_model(model_name)
        export_model(model, args.outdir, args.hash_length)

if __name__ == "__main__":
    main()




import re
import os
from pathlib import Path
from datetime import datetime
from dateutil.relativedelta import relativedelta
from typing import Dict, Any, Union
from collections import defaultdict

def convert_bytes(quant: float, units_in: str, units_out: str) -> float:
    """
    Converts quantity from one byte unit to another.
    
    :param quant: The quantity to convert.
    :param units_in: The byte unit of the input quantity, e.g., "GB".
    :param units_out: The byte unit of the output quantity, e.g., "MB".
    :return: The converted quantity.
    """

    byte_units = {
        "BIT": 8*1024**3,
        'B': 1024**3,
        'KB': 1024**2,
        'MB': 1024,
        'GB': 1,
        'TB': 1/1024,
        'PB': 1/1024**2,
    }
    return convert_unit(quant, units_in, units_out, byte_units)

def convert_time(quant: float, units_in: str, units_out: str) -> float:
    """
    Converts quantity from one time unit to another.
    
    :param quant: The quantity to convert.
    :param units_in: The time unit of the input quantity, e.g., "days".
    :param units_out: The time unit of the output quantity, e.g., "hours".
    :return: The converted quantity.
    """

    # Let's center it around hours
    time_units = {
        'y': 365*24,
        'month': 30*24,
        'w': 7*24,
        'd': 24,
        'h': 1,
        'm': 1/60,
        's': 1/3600,
        'ms': 1/3600000,
        'us': 1/3600000000,
    }
    return convert_unit(quant, units_in, units_out, time_units)

def convert_unit(quant: float,
                 units_in: str,
                 units_out: str,
                 units: Dict[str, float]) -> float:
    """
    Converts quantity from one byte unit to another.

    The `units` dictionary should all express the same amount of quantity,
    e.g., 1 kg = 1000 g, or 1 km = 1*10^3 m = 1*10^6 mm.
    
    :param quant: The quantity to convert.
    :param units_in: The unit of the input quantity, e.g., "GB".
    :param units_out: The unit of the output quantity, e.g., "MB".
    :param units: A dictionary mapping unit names to conversion factors.
    :return: The converted quantity.
    :raises ValueError: If the specified units are not recognized.
    """

    if units_in not in units:
        raise ValueError(f"Unknown input unit: {units_in}")
    if units_out not in units:
        raise ValueError(f"Unknown output unit: {units_out}")
    return quant / units[units_in] * units[units_out]

def get_file_info(path: Union[Path,str], size_units='B') -> Dict[str, Any]:
    """
    Retrieves detailed information about a file, including creation and modification times.

    :param path: Path to the file.
    :param size_units: The units to use for the file size.
    :return: Dictionary containing file metadata.
    """
    if isinstance(path, str):
        path = Path(path)

    stat = path.stat()   
    info = {
        'file_name': path.name,
        'file_path': str(path),
        'file_size': convert_bytes(stat.st_size, 'B', size_units),
        'file_size_units': size_units,
        'last_modification': datetime.fromtimestamp(stat.st_mtime).isoformat(),
        'metadata_change_time': datetime.fromtimestamp(stat.st_ctime).isoformat()
    }
    return info

def parse_duration(duration: str) -> relativedelta:
    """
    Parses a string representing a duration and returns a tuple of a
    relativedelta object and a timedelta object.

    The relativedelta object can be used for precise date arithmetic, while the
    timedelta object can be used to get the total duration in a specific unit.

    Example usage:

        (duration, total_duration) = parse_duration("1 month, 2 weeks, and 3 days")
        occured = datetime.now() - duration
        total_days = total_duration.days

    :param duration: A string representing a duration, e.g., "1 month, 2 weeks, and 3 days".
    :return: A tuple of a relativedelta object and a timedelta object representing the duration.
    :raises ValueError: If the input format is invalid.
    """
    units = ['day', 'week', 'month', 'year', 'hour', 'minute', 'second']
    units_pattern = '|'.join(units)
    pattern = re.compile(rf'(\d+)\s*({units_pattern})s?', re.IGNORECASE)
    matches = pattern.findall(duration)
    
    if not matches:
        raise ValueError("Invalid input format. Expected something like '1 month and 2 weeks'")
    
    relativedelta_kwargs = defaultdict(int)
    for value, unit in matches:
        value = int(value)
        unit = unit.lower().rstrip('s')  # Normalize unit to singular form
        # Accumulate values for each unit
        relativedelta_kwargs[unit] += value

    # add 's' to each unit for relativedelta
    relativedelta_kwargs = {f"{unit}s": value for unit, value in relativedelta_kwargs.items()}
    rd = relativedelta(**relativedelta_kwargs).normalized()
    now = datetime.now()
    future = now + rd
    td = future - now
    return rd, td



from setuptools import setup, find_packages
import pathlib

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.md").read_text()

setup(
    name='ollama_data_tools',
    version='0.1.1',
    description='Tools for working with Ollama model data',
    author='Alex Towell',
    author_email='lex@metafunctor.com',
    long_description=README,
    long_description_content_type="text/markdown",
    url='https://github.com/queelius/ollama_data_tools',
    packages=find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=[
        # other dependencies...
    ],
    entry_points={
        'console_scripts': [
            'ollama_data_export=ollama_data_tools.ollama_data_export:main',
            'ollama_data_adapter=ollama_data_tools.ollama_data_adapter:main',
            'ollama_data_query=ollama_data_tools.ollama_data_query:main',
            # other scripts...
        ],
    },
)



import subprocess
from pathlib import Path
import re
from datetime import datetime
from typing import Dict, List, Any
from ollama_data_tools import conversion_tools as ct

def get_schema() -> List[Dict[str, Any]]:
    """
    Returns the schema for the model information.
    
    :return: A list of dictionaries representing the schema.
    """
    return [{
        'name': '<str>',
        'model_params': '<dict>',
        'system_message': '<list[str]>',
        'total_weights_size': '<float>',
        'total_weights_size_units': '<str>',
        'template': '<list[str]>',
        'modelfile': '<str|None>',
        'last_modified': '<str>',
        'age': {
            'days': '<int>',
            'years': '<int>',
            'months': '<int>',
            'weeks': '<int>',
            'seconds': '<int>'
        },
        'weights': [{
            'hash': '<str>',
            'path': '<str>',
            'file_name': '<str>',
            'dir': '<str>',
            'file_size': '<float>',
            'file_size_units': '<str>',
            'last_modified': '<str>',
            'metadata_modified': '<str>'
        }]
    }]

def run_ollama(args: List[str]) -> str:
    """
    Execute `ollama` command with arguments and returns the output.
    
    :param args: The arguments to use.
    :return: The output of the command.
    :raises RuntimeError: If the command fails.
    """
    
    result = subprocess.run(['ollama'] + args, capture_output=True, text=True)
    if result.returncode != 0:
        raise RuntimeError(f"ollama with args [{ ''.join(args) }] failed with error: {result.stderr}")
    return result.stdout

def get_models(exclude_keys = None) -> List[Dict[str, Any]]:
    """
    Generates a list of dictionaries containing information about all models in Ollama.

    :param exclude_keys: A list of keys to exclude from the output.
    
    :return: A list of dictionaries with model information.
    """
    output = run_ollama(['list'])
    lines = output.splitlines()
    lines = [line for line in lines if line.strip()]
    if lines and lines[0].startswith('NAME'):
        lines.pop(0)

    models = []
    for line in lines:
        parts = line.split()
        model_name = parts[0]
        #weights_size_gb = convert_bytes(float(parts[2]), parts[3], 'GB')
        dur, delta = ct.parse_duration(parts[4] + ' ' + parts[5])
        
        #weight_info['size_units'] = 'GB'

        weight_infos = []
        weight_paths = get_weights_path(model_name)
        for weight_path in weight_paths:
            weight_info = ct.get_file_info(weight_path)
            match = re.search(r'.*/sha256-([a-f0-9]{64})',
                      str(weight_path), re.IGNORECASE)
            weight_info['hash'] = match.group(1) if match else None
            weight_info['dir'] = str(weight_path.parent)
            weight_infos.append(weight_info)

        total_weights_size = sum([info['file_size'] if 'file_size' in info else 0
                                  for info in weight_infos])        
        models.append({
            'name': model_name,
            'last_modified': (datetime.now() - delta).isoformat(),
            'age': {
                "days": dur.days,
                "seconds": dur.seconds,
                "years": dur.years,
                "months": dur.months,
                "weeks": dur.weeks,
                "hours": dur.hours,
                "minutes": dur.minutes,
            },
            'model_params': get_model_params(model_name),
            'system_message': get_model_system(model_name),
            'template': get_model_template(model_name),
            'modelfile': get_modelfile(model_name),
            'total_weights_size': ct.convert_bytes(total_weights_size, 'B', 'GB'),
            'total_weights_size_units': 'GB',
            #'total_weights_size_alternate': weights_size_gb,
            'weights': weight_infos
        })
    return models

def get_model_weights_license(model_name: str) -> str:
    """
    Fetches the license type of a model's weights using `ollama show --license`.

    :param model_name: The name of the model.
    :return: The license type of the model's weights, or None if the command fails.
    """
    return run_ollama(['show', model_name, '--license'])

def get_modelfile(model_name: str) -> str:
    """
    Fetches the model's modelfile using `ollama show --modelfile`.

    :param model_name: The name of the model.
    :return: The modelfile contents, or None if the command fails.
    """
    return run_ollama(['show', model_name, '--modelfile'])

def get_weights_hash(model_name: str) -> str:
    """
    Retrieves the SHA256 hashes of a model's weights.

    :param model_name: The name of the model.
    :return: A list of SHA256 hashes.
    """
    modelfile = get_modelfile(model_name)
    return re.findall(r'\s*FROM .*/sha256-([a-f0-9]{64})',
                      modelfile, re.IGNORECASE | re.MULTILINE)

def get_weights_path(model_name: str) -> Dict[str, str]:
    """
    Fetches the path and filename of a model's weights.

    :param model_name: The name of the model.
    :return: A dictionary containing the path and filename, or None if not found.
    """
    modelfile = get_modelfile(model_name)
    match = re.findall(r'^\s*FROM (.+)', modelfile, re.IGNORECASE | re.MULTILINE)
    return [Path(filename) for filename in match]
    
def get_model_system(model_name: str) -> str:
    """
    Fetches the system message of a model using `ollama show --system`.

    :param model_name: The name of the model.
    :return: The system message, or None if the command fails.
    """
    return run_ollama(['show', model_name, '--system'])

def get_model_template(model_name: str) -> List[str]:
    """
    Fetches the template of a model using `ollama show --template`.

    :param model_name: The name of the model.
    :return: A list of template lines, or None if the command fails.
    """
    output = run_ollama(['show', model_name, '--template'])
    return [line.strip() for line in output.splitlines() if line.strip()]

def get_model_params(model_name: str) -> Dict[str, str]:
    """
    Fetches the parameters of a model using `ollama show --parameters`.

    :param model_name: The name of the model.
    :return: A dictionary of model parameters, or None if the command fails.
    """
    output = run_ollama(['show', model_name, '--parameters'])
    lines = output.splitlines()
    params = {}

    regex = r'(?:"[^"]*"|[^"\s]+)+'
    for line in lines:
        if line.strip():
            key, value = re.findall(regex, line)
            params[key.strip()] = value.strip()
    return params



import json
import re
import jmespath

def validate_json(x):
    """
    Validate that an object is JSON-compatible.

    :param x: The object to validate.
    :raises ValueError: If the object is not JSON-compatible.
    """
    try:
        json.dumps(x)
    except (TypeError, OverflowError):
        raise ValueError("Object is not JSON-compatible")

def regex_path_matcher(data, regex, path='@', elemwise=True):
    """
    Match a regular expression to the JSON data on a JMESPath query.
    We apply he regex matcher to the view of the data that the JMESPath
    query returns, or if element-wise, we apply the matcher
    to each sub-element of data, e.g., if we give a list of dictionaries,
    we apply the matcher to each dictionary.

    We serialize the JSON view of the data to a string before applying the
    regex pattern.

    :param data: The JSON data to match the regex pattern against.
    :param regex: The regex pattern.
    :param path: The JMESPath query that creates the view to apply
                 the matcher to.
    :param elemwise: Whether to apply the regex matcher to each element
                     of data as opposed to the entire data object.

    :return: JSON (dict) object representing a filtered view of the top-level
                JSON output.
    """

    validate_json(data)
    regex = re.compile(regex)
    if elemwise:
        if isinstance(data, list):
            return [m for m in data if regex.search(str(jmespath.search(path, m)))]
        elif isinstance(data, dict):
            return {k: v for k, v in data.items() if regex.search(str(jmespath.search(path, {k: v})))}

    return data if regex.search(str(jmespath.search(path, data))) else None
    
